/*
 *  Appindex browser
 *  Copyright (C) 1999 Martin Hinner <mhi@penguin.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <curses.h>
#include <string.h>

#include "appindex.h"

static void
draw_appscreen ()
{
  attrset (COLOR_PAIR (CP_HEADER));
  mvprintw (0, 0, "appindex browser                                             ");
  mvprintw (0, COLS - (strlen (gendate) + 6), "Date: %s", gendate);

  attrset (COLOR_PAIR (CP_KEY));
  mvprintw (LINES - 1, 0, "q,<bs>");
  attrset (COLOR_PAIR (CP_KEYTEXT));
  printw ("-back ");

  attrset (COLOR_PAIR (CP_KEY));
  printw ("h");
  attrset (COLOR_PAIR (CP_KEYTEXT));
  printw ("-homepage ");

  attrset (COLOR_PAIR (CP_KEY));
  printw ("d");
  attrset (COLOR_PAIR (CP_KEYTEXT));
  printw ("-download ");

  attrset (COLOR_PAIR (CP_KEY));
  printw ("c");
  attrset (COLOR_PAIR (CP_KEYTEXT));
  printw ("-changelog ");

  attrset (COLOR_PAIR (CP_KEY));
  printw ("f");
  attrset (COLOR_PAIR (CP_KEYTEXT));
  printw ("-freshmeat ");

  attrset (COLOR_PAIR (CP_KEY));
  printw ("r");
  attrset (COLOR_PAIR (CP_KEYTEXT));
  printw ("-rpm ");

  attrset (COLOR_PAIR (CP_KEY));
  printw ("b");
  attrset (COLOR_PAIR (CP_KEYTEXT));
  printw ("-deb ");

  printw ("          ");
}

/*
   Draws items on screen[1,LINES-1].
 */
static void
draw_appitems (struct app *current, struct app *selected)
{
  int x;
  char line[0x100];

  for (x = 1; x < LINES - INFO_LINES - 1; x++)
    {
      if (current)
	{
	  if (current == selected)
	    attrset (COLOR_PAIR (CP_SELECTED));
	  else
	    attrset (COLOR_PAIR (CP_NORMAL));
	  strncpy (line, current->name, 0x40);
	  if (strlen (line) > 31)
	    {
	      line[27] = 0;
	      strcat (line, "...");
	    }

	  mvprintw (x, 0, "  %-31.31s", line);

	  strncpy (line, current->desc, 0x40);
	  if (strlen (line) > COLS - 35)
	    {
	      line[COLS - 38] = 0;
	      strcat (line, "...");
	    }
	  printw (" %-*.*s ", COLS - 35, COLS - 35, line);

	  current = current->next;
	}
      else
	{
	  attrset (COLOR_PAIR (CP_NORMAL));
	  mvprintw (x, 0, "%*.*s", COLS - 2, COLS - 2, "");	/* Aaaargh! FIXME */
	}
    }

}

static void
draw_appinfo (struct app *app, char *catname)
{
  char line[0x100];
  int x;
  int d2idx;			/* Second description line index */

  attrset (COLOR_PAIR (CP_HEADER));
  strncpy (line, catname, 0x100);
  strncat (line, ":", 0x100 - strlen (line));
  strncat (line, app->name, 0x100 - strlen (line));
  mvprintw (LINES - INFO_LINES - 1, 0, "%-*.*s", COLS, COLS, line);

  attrset (COLOR_PAIR (CP_INFOKEY) | A_BOLD);
  mvprintw (LINES - INFO_LINES, 0, "License:     ");
  attrset (COLOR_PAIR (CP_INFOVAL));
  printw ("%-*.*s", COLS - 56, COLS - 56, app->license);
  attrset (COLOR_PAIR (CP_INFOKEY) | A_BOLD);
  printw ("  Stable: ");
  attrset (COLOR_PAIR (CP_INFOVAL));
  printw ("%-8.8s", app->stable);
  attrset (COLOR_PAIR (CP_INFOKEY) | A_BOLD);
  printw ("  Development: ");
  attrset (COLOR_PAIR (CP_INFOVAL));
  printw ("%-8.8s  ", app->devel);

  attrset (COLOR_PAIR (CP_INFOKEY) | A_BOLD);
  mvprintw (LINES - INFO_LINES + 1, 0, "Homepage:    ");
  attrset (COLOR_PAIR (CP_INFOVAL));
  printw ("%-*.*s", COLS - 13, COLS - 13, app->homepage);

  attrset (COLOR_PAIR (CP_INFOKEY) | A_BOLD);
  mvprintw (LINES - INFO_LINES + 2, 0, "Download:    ");
  attrset (COLOR_PAIR (CP_INFOVAL));
  printw ("%-*.*s", COLS - 13, COLS - 13, app->download);

  attrset (COLOR_PAIR (CP_INFOKEY) | A_BOLD);
  mvprintw (LINES - INFO_LINES + 3, 0, "ChangeLog:   ");
  attrset (COLOR_PAIR (CP_INFOVAL));
  printw ("%-*.*s", COLS - 13, COLS - 13, app->changelog);


  attrset (COLOR_PAIR (CP_INFOKEY) | A_BOLD);
  mvprintw (LINES - INFO_LINES + 4, 0, "Deb package: ");
  attrset (COLOR_PAIR (CP_INFOVAL));
  printw ("%-*.*s", COLS - 13, COLS - 13, app->deb);

  attrset (COLOR_PAIR (CP_INFOKEY) | A_BOLD);
  mvprintw (LINES - INFO_LINES + 5, 0, "RPM package: ");
  attrset (COLOR_PAIR (CP_INFOVAL));
  printw ("%-*.*s", COLS - 13, COLS - 13, app->rpm);


  attrset (COLOR_PAIR (CP_INFOKEY) | A_BOLD);
  mvprintw (LINES - INFO_LINES + 6, 0, "Link:        ");
  attrset (COLOR_PAIR (CP_INFOVAL));
  printw ("%-*.*s", COLS - 13, COLS - 13, app->link);


  if (strlen (app->desc) > COLS - 13)
    {
      d2idx = 0;
      for (x = COLS - 13; (x > COLS - 23) && (d2idx == 0); x--)
	if (app->desc[x] == ' ' || app->desc[x] == '\t')
	  d2idx = x;
      if (d2idx == 0)
	d2idx = COLS - 23;
    }
  else
    {
      d2idx = strlen (app->desc);
    }
  attrset (COLOR_PAIR (CP_INFOKEY) | A_BOLD);
  mvprintw (LINES - INFO_LINES + 7, 0, "Description: ");
  attrset (COLOR_PAIR (CP_INFOVAL));
  printw ("%-*.*s", d2idx, d2idx, app->desc);
  for (x = d2idx; x < COLS - 13; x++)	/* fill with spaces */
    printw (" ");
  while (app->desc[d2idx] == ' ')
    d2idx++;
  attrset (COLOR_PAIR (CP_INFOKEY) | A_BOLD);
  mvprintw (LINES - INFO_LINES + 8, 0, "             ");
  attrset (COLOR_PAIR (CP_INFOVAL));
  printw ("%-*.*s", COLS - 13, COLS - 13, app->desc + d2idx);
}


void
app_selection (struct app *app, char *catname, int appno)
{
  struct app *cur, *first;
  int curno, totalno;
  int x;

  cur = app;
  first = app;
  curno = 0;
  totalno = 0;

  draw_appscreen ();

  while (1)
    {
      draw_appitems (first, cur);
      draw_appinfo (cur, catname);
      refresh ();
      switch (getch ())
	{
	case 'q':
	case 'Q':
	case KEY_BACKSPACE:
	  return;
	case KEY_DOWN:
	case 'j':
	case 'J':
	  if (cur->next)
	    {
	      if (curno < LINES - INFO_LINES - 3)
		/* Seek to next item _ON_SCREEN */
		curno++;
	      else
		first = first->next;
	      cur = cur->next;
	      totalno++;
	    }
	  break;
	case KEY_UP:
	case 'k':
	case 'K':
	  if (cur->prev)
	    {
	      if (curno > 0)
		curno--;
	      else
		first = first->prev;
	      cur = cur->prev;
	      totalno--;
	    }
	  break;
	case KEY_HOME:
	  cur = first = app;
	  curno = totalno = 0;
	  break;
	case KEY_END:
	  if (appno > LINES - INFO_LINES - 3)
	    {
	      curno = LINES - INFO_LINES - 3;
	      totalno = appno;
	      while (cur->next)
		cur = cur->next;
	      first = cur;
	      for (x = 0; x < LINES - INFO_LINES - 3; x++)
		first = first->prev;
	    }
	  else
	    {
	      first = app;
	      curno = totalno = appno;
	      while (cur->next)
		cur = cur->next;
	    }
	  break;
	case KEY_NPAGE:
	  if (appno > LINES - INFO_LINES - 3)
	    {
	      if (curno != LINES - INFO_LINES - 3)
		{		/* skip to end of screen */
		  while (curno != LINES - INFO_LINES - 3)
		    {
		      cur = cur->next;
		      curno++;
		      totalno++;
		    }
		}
	      else
		/* page down */
		{
		  for (x = 0; x < LINES - INFO_LINES - 3; x++)
		    if (cur->next)
		      {
			cur = cur->next;
			first = first->next;
			totalno++;
		      }
		}
	    }
	  else
	    {
	      first = app;
	      curno = totalno = appno;
	      while (cur->next)
		cur = cur->next;
	    }
	  break;
	case KEY_PPAGE:
	  if (appno > LINES - INFO_LINES - 3)
	    {
	      if (curno != 0)
		{
		  while (curno != 0)
		    {
		      cur = cur->prev;
		      curno--;
		      totalno--;
		    }
		}
	      else
		{
		  for (x = 0; x < LINES - INFO_LINES - 3; x++)
		    if (cur->prev)
		      {
			cur = cur->prev;
			first = first->prev;
			totalno--;
		      }
		}
	    }
	  else
	    {
	      cur = first = app;
	      curno = totalno = 0;
	    }
	  break;
	case 'h':
	case 'H':
	  if (cur->homepage[0])
	    run_browser (cur->homepage);
	  else
	    error ("No homepage URL specified!");
	  clear ();
	  draw_appscreen ();
	  refresh ();
	  break;
	case 'd':
	case 'D':
	  if (cur->download[0])
	    run_browser (cur->download);
	  else
	    error ("No download URL specified!");
	  clear ();
	  draw_appscreen ();
	  refresh ();
	  break;
	case 'f':
	case 'F':
	  if (cur->link[0])
	    run_browser (cur->link);
	  else
	    error ("No freshmeat link URL specified!");
	  clear ();
	  draw_appscreen ();
	  refresh ();
	  break;
	case 'r':
	case 'R':
	  if (cur->rpm[0])
	    run_browser (cur->rpm);
	  else
	    error ("No .rpm package specified!");
	  clear ();
	  draw_appscreen ();
	  refresh ();
	  break;
	case 'b':
	case 'B':
	  if (cur->deb[0])
	    run_browser (cur->deb);
	  else
	    error ("No .deb package specified!");
	  clear ();
	  draw_appscreen ();
	  refresh ();
	  break;
	case 'c':
	case 'C':
	  if (cur->changelog[0])
	    run_browser (cur->changelog);
	  else
	    error ("No changelog URL specified!");
	  clear ();
	  draw_appscreen ();
	  refresh ();
	  break;
	}
    }
}
