/*
 *  Appindex browser
 *  Copyright (C) 1999 Martin Hinner <mhi@penguin.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "appindex.h"

int total_apps;

char *header[] =
{
  "",
  "syntax:",
  "name",
  "stable",
  "devel",
  "section/category",
  "license",
  "homepage",
  "download",
  "changelog",
  "deb",
  "rpm",
  "link",
  "oneliner",
  0
};

int
skip_to_next_app (FILE * f)
{
  char *line;

  line = (char *) malloc (LINE_LEN);
  if (!line)
    die ("Not enough memory!");

  while (fgets (line, LINE_LEN, f))
    {
      if (!strcmp (line, "%%\n"))
	{
	  free (line);
	  return 1;
	}
    }

  free (line);
  return 0;
}

void
add_app (char *category, struct app *app)
{
  struct category *cat, *c, *pcat;


  if (categories)
    if (strcmp (category, categories->name) >= 0)
      goto scan;
  cat = (struct category *) malloc (sizeof (struct category));
  if (!cat)
    die ("Not enough memory");
  cat->name = strdup (category);
  if (!cat->name)
    die ("Not enough memory");
  app->next = 0;
  cat->apps = app;
  cat->next = categories;
  categories = cat;
  return;


scan:
  pcat = 0;
  c = categories;
  while (c)
    {
      if (!strcmp (c->name, category))
	{
	  app->next = c->apps;
	  c->apps = app;
	  return;
	}
      if (c->next)
	{
	  if ((strcmp (c->name, category) < 0)
	      && (strcmp (c->next->name, category) > 0))
	    {
	      cat = (struct category *) malloc (sizeof (struct category));
	      if (!cat)
		die ("Not enough memory");
	      cat->name = strdup (category);
	      if (!cat->name)
		die ("Not enough memory");
	      app->next = 0;
	      cat->apps = app;
	      cat->next = c->next;
	      c->next = cat;
	      return;

	    }
	}
      pcat = c;
      c = c->next;
    }
  cat = (struct category *) malloc (sizeof (struct category));
  if (!cat)
    die ("Not enough memory");
  cat->name = strdup (category);
  if (!cat->name)
    die ("Not enough memory");
  app->next = 0;
  cat->apps = app;
  cat->next = 0;
  pcat->next = cat;
  return;
}

void
load_file (FILE * f)
{
  char *line;
  int n;
  struct app *app;
  char *category;

  skip_to_next_app (f);

  line = (char *) malloc (LINE_LEN);
  if (!line)
    die ("Not enough memory!");

  if (!fgets (line, LINE_LEN, f))
    die ("Invalid file format");

  if (line[strlen (line) - 1] == '\n')
    line[strlen (line) - 1] = 0;
  if (strncmp (line, "Date of generation: ", 20))
    die ("Invalid file format");
  strcpy (gendate, line + 20);

  n = 0;
  while (header[n])
    {
      if (!fgets (line, LINE_LEN, f))
	die ("Invalid file format");

      if (line[strlen (line) - 1] == '\n')
	line[strlen (line) - 1] = 0;
      if (strcmp (line, header[n]))
	die ("Invalid file format");
      n++;
    }

  while (skip_to_next_app (f))
    {
      app = (struct app *) malloc (sizeof (struct app));
      if (!app)
	die ("Not enough memory");

#define GET_ITEM(x) \
      if (!fgets(line,LINE_LEN,f)) \
        die("Broken data file"); \
      if (line[strlen (line) - 1] == '\n') \
	line[strlen (line) - 1] = 0; \
      x = strdup(line); \
      if (!x) \
       die("Not enough memory");

      GET_ITEM (app->name);
      GET_ITEM (app->stable);
      GET_ITEM (app->devel);
      GET_ITEM (category);
      GET_ITEM (app->license);
      GET_ITEM (app->homepage);
      GET_ITEM (app->download);
      GET_ITEM (app->changelog);
      GET_ITEM (app->deb);
      GET_ITEM (app->rpm);
      GET_ITEM (app->link);
      GET_ITEM (app->desc);
#undef GET_ITEM
      total_apps++;
      add_app (category, app);
      loadstat (total_apps);
      free (category);
    }


  free (line);
}

void
load_appindex (char *file)
{
  FILE *f;

  total_apps = 0;
  if (file)
    {
      f = fopen (file, "rt");
      if (f)
	{
	  load_file (f);
	  fclose (f);
	  return;
	}
      else
	die ("Cannot open appindex data file");
    }

  f = fopen ("~/.appindex.txt", "rt");
  if (f)
    {
      load_file (f);
      fclose (f);
      return;
    }

  f = fopen ("/usr/local/lib/appindex/appindex.txt", "rt");
  if (f)
    {
      load_file (f);
      fclose (f);
      return;
    }

  /* TODO: load gzipped appindex from /usr/lib/appindex/appindex.txt.gz */


}

int
relink_app (struct app *apps)
{
  struct app *ap, *a;
  int n;

  a = apps;
  ap = 0;
  n = 0;
  while (a)
    {
      a->prev = ap;
      ap = a;
      a = a->next;
      n++;
    }
  return n;
}

void
relink_cat ()
{
  struct category *cp, *c;
  int n;

  c = categories;
  cp = 0;
  n = 0;
  while (c)
    {
      c->prev = cp;
      c->appno = relink_app (c->apps);
      cp = c;
      c = c->next;
      n++;
    }
  catno = n;
}
