/*
 *  Appindex browser
 *  Copyright (C) 1999 Martin Hinner <mhi@penguin.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <curses.h>
#include <signal.h>
#include <getopt.h>

#include "appindex.h"

static char *usage =
"Usage: %s [OPTION] [FILE]\n"
"\n"
"   -c [file]        Configuration file instead of default\n"
"   -h               Show this help\n"
"\n"
"Report bugs to <mhi@penguin.cz>\n";

char gendate[LINE_LEN];
struct category *categories;
int catno, appno;

void
trap (int sig)
{

  move (LINES - 1, 0);
  clrtoeol ();
  refresh ();

  endwin ();
  exit (sig);
}

int
main (int argc, char **argv)
{
  char *conf_file;
  char c;

  conf_file = 0;

  while ((c = getopt (argc, argv, "c:h")) != -1)
    {
      switch (c)
	{
	case 'c':
	  conf_file = optarg;
	  break;
	case 'h':
	  fprintf (stderr, usage, argv[0]);
	  exit (0);
	default:
	  fprintf (stderr, usage, argv[0]);
	  exit (1);
	}
    }


  /* Parse configuration file */
  null_config ();
  if (conf_file)
    parse_config (conf_file);
  else
    {
      parse_config ("~/.appindexrc");
      parse_config ("/usr/local/etc/appindex.conf");
      parse_config ("/etc/appindex.conf");
    }
  if (configure () < 0)
    {
      return 1;
    }

  argc -= optind;
  argv += optind;

  /* initialize curses library */
  initscr ();
  noecho ();
  keypad (stdscr, TRUE);
  start_color ();
  cbreak ();

  /* initialize colors */
  init_pair (CP_HEADER, colors[CP_HEADER] >> 4, colors[CP_HEADER] & 0xF);
  init_pair (CP_NORMAL, colors[CP_NORMAL] >> 4, colors[CP_NORMAL] & 0xF);
  init_pair (CP_SELECTED, colors[CP_SELECTED] >> 4, colors[CP_SELECTED] & 0xF);
  init_pair (CP_KEY, colors[CP_KEY] >> 4, colors[CP_KEY] & 0xF);
  init_pair (CP_KEYTEXT, colors[CP_KEYTEXT] >> 4, colors[CP_KEYTEXT] & 0xF);
  init_pair (CP_INFOKEY, colors[CP_INFOKEY] >> 4, colors[CP_INFOKEY] & 0xF);
  init_pair (CP_INFOVAL, colors[CP_INFOVAL] >> 4, colors[CP_INFOVAL] & 0xF);
  init_pair (CP_ERROR, colors[CP_ERROR] >> 4, colors[CP_ERROR] & 0xF);

  categories = 0;
  strcpy (gendate, "(unknown)");

  signal (SIGINT, trap);

  attrset (COLOR_PAIR (CP_HEADER));
  mvprintw (0, 0, "%-*.*s", COLS, COLS,
	    "appindex browser (c) 1999 Martin Hinner <mhi@penguin.cz>");
  refresh ();

  load_appindex (argc ? argv[0] : NULL);
  relink_cat ();

  category_selection (categories);

  trap (0);
  return 0;
}
